/*
 * main.cpp
 *
 *  Created on: Sep 7, 2013
 *      Author: nico
 */

#include "global.h"



wxSerialPort SerialPort;


//hold the amount of charge
double charge_accu;
double watt_accu;
int charge_time;	//duration of the discharge

//Remember these for sending settings commands. Filled automatically by get_status
int max_current;
int max_power;


/*************************************************************************
  Communication
*************************************************************************/

#define MESSAGE_LENGTH	26

int send_command(unsigned char *cmd)
{
	//send cmd to DC load and return answer in cmd
	int len;
	int loop;
	unsigned char *ptr;

	int bufpos;
	char a;

	if (SerialPort.IsOpen()==false) return -1;

	//Make sure there is no data in the serial port buffer before we send
	//the command.
	while (SerialPort.Read(&a, 1)>0);
	int sum=0;

	//fill in the header and address
	cmd[0]=0xaa;
	cmd[1]=0;

	for(loop=0; loop< (MESSAGE_LENGTH-1); loop++)
		{
		sum+=cmd[loop];
		}
	cmd[loop]=sum & 0xff;

	//send command
	if (SerialPort.Write((char*) cmd, MESSAGE_LENGTH)<0)
		{
		SerialPort.Close();
		return -1;	//exit if failure
		}


	//read data (wait 400ms max for answer)
	loop=0;
	ptr=cmd;
	bufpos=0;
	while ( (loop<20) && (bufpos<(MESSAGE_LENGTH-1)))
		{
		usleep(20000);	//sleep at least 20ms
		len=SerialPort.Read((char*) ptr, MESSAGE_LENGTH - bufpos-1);
		if (len>0)
			{
			bufpos+=len;
			ptr+=len;
			}
		loop++;
		}

	if (bufpos!=(MESSAGE_LENGTH-1)) return -1;

	return 1;
}


int set_current(double amps)
{
	unsigned char cmd[MESSAGE_LENGTH];
	int val;
	memset(cmd, 0, sizeof(cmd));

	cmd[2]=0x90;	//set parameters
	val=amps * 1000;

	//set max current and power
	cmd[3]=max_current & 0xff;
	cmd[4]=(max_current>>8) & 0xff;
	cmd[5]=max_power & 0xff;
	cmd[6]=(max_power>>8) & 0xff;

	cmd[8]=1;	//set current
	cmd[9]=val & 0xff;
	cmd[10]=(val>>8) & 0xff;

	return send_command(cmd);
}


int control_load(bool locked, bool load_on)
{
	unsigned char cmd[MESSAGE_LENGTH];
	memset(cmd, 0, sizeof(cmd));

	cmd[2]=0x92;
	if (locked==true) cmd[3]|=2;
	if (load_on==true) cmd[3]|=1;

	return send_command(cmd);
}


int get_status(double &amps, double &volts)
{
	int amp, volt;
	unsigned char cmd[MESSAGE_LENGTH];
	memset(cmd, 0, sizeof(cmd));

	cmd[2]=0x91;
	amps=0;
	volts=0;

	if (send_command(cmd)<0) return -1;

	volt=cmd[5] + (cmd[6] <<8) + (cmd[7] <<16) + (cmd[8] <<24);
	amp=cmd[3] + (cmd[4]<<8);
	volts=volt/1000.0;
	amps=amp/1000.0;


	max_current=cmd[11] + (cmd[12]<<8);
	max_power=cmd[13] + (cmd[14]<<8);

	//debug
	/*
	int i;
	for(i=0; i<25; i++)
		{
		printf("%.2d: %d\n", i+1, cmd[i]);
		}

	printf("volt: %f amp: %f\n", volts, amps);
//*/
	return 1;
}

/*************************************************************************
  Main
*************************************************************************/



int exit_program=0;

void sig_handler(int signum)
{
	exit_program=1;
}



int main(int argc, char **argv)
{

	/*
	 * port amperes min_voltage go
	 *
	 */
	int go=0;
	int csv=0;
	double ampere;
	double min_volt;
	time_t last_time=0;
	FILE *fp=NULL;


  signal(SIGINT, sig_handler);

	if (argc<4)
		{
		printf("Usage: %s <port> <ampere> <stop voltage> [go] [filename]\n", argv[0]);
		printf(" port = serial port (/dev/ttyUSB  \\COM3:)\n");
		printf(" ampere = discharge current\n");
		printf(" stop voltage = voltage when the discharge must stop\n");
		printf(" go (optional) = without go %s does a dry run (test connectivity) but doesn't discharge\n", argv[0]);
		printf(" filename (optional) = create a csv file with the discharge values (1 line per minute)\n");
		printf("  Example: /dev/ttyUSB0  1.5  10.4\n");
		return 1;
		}

	ampere=atof(argv[2]);
	min_volt=atof(argv[3]);

	wxSerialPort_DCS dcs;   //dcs structure for settings
	dcs.baud=wxBAUD_9600;
	dcs.parity=wxPARITY_NONE;
	dcs.stopbits=1;
	dcs.wordlen=8;
	dcs.rtscts=false;
	dcs.xonxoff=false;



	SerialPort.Open(argv[1], &dcs);
	if (SerialPort.IsOpen()==false)
		{
		printf("Cannot open %s\n", argv[1]);
		goto error;
		}

	control_load(true, false);

	double a,b;
	if (get_status(a, b)<0)
		{
		//only get_status returns data
		printf("DC load not responding\n");
		goto error;
		}


	if ((argc>=5) && (strcmp(argv[4], "go")==0)) go=1;
	if ((argc>=6) && (strlen(argv[5])!=0)) csv=1;

	set_current(ampere);
	int count;



	if (go==1)
		{
		count=0;

		if (csv!=0)
			{
			fp=fopen(argv[5], "wb");	//wb to prevent \n being expaned to \r\n on Windows
			if (fp!=NULL)
				{
				fprintf(fp, "timestamp;volt;amp;charge;Wh\r\n");
				fclose(fp);
				}

			}


		control_load(true, true);

		//do the discharge
		while(exit_program==0)
			{

			if (time(NULL)!=last_time)
				{
				last_time=time(NULL);

				double amps, volts;
				if (get_status(amps, volts)<0)
					{
					printf("Cannot read DC load. Exiting\n");
					goto error;
					}

				if (volts<min_volt)
					{
					break;
					}
				charge_accu+=amps;
				watt_accu+=volts*amps;
				charge_time++;

				count++;
				if (count==60)
					{
					//once per minute
					printf("charge: %.2f\n", charge_accu/3600.0);

					if (csv!=0)
						{
						fp=fopen(argv[5], "ab");	//wb to prevent \n being expaned to \r\n on Windows
						if (fp!=NULL)
							{
							fprintf(fp, "%ld;%.2f;%.2f;%.2f;%.2f\r\n", time(NULL), volts, amps, charge_accu/3600.0, watt_accu/3600.0);
							fclose(fp);
							}

						}

					count=0;
					}

				}

			usleep(100000);
			}
		}


	printf("Time: %d:%d Total charge: %.2fAh energy: %.2fWh\n", charge_time/60, charge_time % 60, charge_accu/3600.0, watt_accu/3600.0);

error:
	//make an attempt to leave the DC load in a known state (output off and front panel enabled)
	if (SerialPort.IsOpen()==false) SerialPort.Open(argv[1], &dcs);

	control_load(true, false);
	control_load(false, false);

	SerialPort.Close();

	printf("Exit\n");

	return 0;

}
